@@ -132,7 +132,9 @@ frail.fit <- function(X,
               frail = double(n.fam),
            ##   mu = double(n),
               convergence = integer(1),
-              fail = integer(1)
+              fail = integer(1),
+              DUP = FALSE,
+              PACKAGE = "eha"
               )  
 
     if (fit$fail){
